﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Search and Remove Rows", 1, ColumnGroupState.ColumnSelected, "Change Values",
    Position = 1, OptionsView = typeof(SearchWithMatchOperationCommandView))]
  public class DeleteRowsWithGivenValueCommand : SearchWithMatchOperationCommandBase {
    private Dictionary<int, IComparable[]> deletedRows;

    [StorableConstructor]
    protected DeleteRowsWithGivenValueCommand(bool deserializing)
      : base(deserializing) {
      deletedRows = new Dictionary<int, IComparable[]>();
    }

    public DeleteRowsWithGivenValueCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      deletedRows = new Dictionary<int, IComparable[]>();
    }

    public override string Description {
      get { return "Delete rows which match given value"; }
    }

    public override void Execute() {
      base.Execute();
      Point[] affectedCells = this.GetAffectedCells();
      int[] rows = affectedCells.Select(c => c.Y).Distinct().OrderBy(x => x).Reverse().ToArray();

      foreach (int rowIndex in rows) {
        deletedRows[rowIndex] = ColumnGroup.GetRow(rowIndex);
        ColumnGroup.DeleteRow(rowIndex);
      }
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      foreach (KeyValuePair<int, IComparable[]> row in deletedRows.OrderBy(x => x.Key))
        ColumnGroup.InsertRow(row.Key, row.Value);
      deletedRows.Clear();
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
